//
//  Copyright (c) 2022 Open Whisper Systems. All rights reserved.
//

#import <SignalServiceKit/BaseModel.h>
#import <SignalServiceKit/ProfileManagerProtocol.h>

NS_ASSUME_NONNULL_BEGIN

typedef void (^OWSUserProfileCompletion)(void);

@class AnyUserProfileFinder;
@class OWSAES256Key;
@class OWSUserProfileBadgeInfo;
@class SDSAnyReadTransaction;
@class SDSAnyWriteTransaction;
@class SignalServiceAddress;
@class UserProfileChanges;

extern NSNotificationName const kNSNotificationNameProfileWhitelistDidChange;
extern NSNotificationName const kNSNotificationNameLocalProfileDidChange;
extern NSNotificationName const kNSNotificationNameLocalProfileKeyDidChange;
extern NSNotificationName const kNSNotificationNameOtherUsersProfileWillChange;
extern NSNotificationName const kNSNotificationNameOtherUsersProfileDidChange;

extern NSString *const kNSNotificationKey_ProfileAddress;
extern NSString *const kNSNotificationKey_ProfileGroupId;

extern NSString *const kLocalProfileInvariantPhoneNumber;

BOOL shouldUpdateStorageServiceForUserProfileWriter(UserProfileWriter userProfileWriter);

NSString *NSStringForUserProfileWriter(UserProfileWriter userProfileWriter);

@protocol OWSMaybeUserProfile
@property (nonatomic, nullable, readonly) OWSUserProfile *userProfileOrNil;
@end

#pragma mark -

@interface OWSUserProfile : BaseModel <OWSMaybeUserProfile>

@property (atomic, readonly) SignalServiceAddress *address;
@property (atomic, readonly, nullable) OWSAES256Key *profileKey;
@property (atomic, readonly, nullable) NSString *unfilteredGivenName;
@property (atomic, readonly, nullable) NSString *givenName;
@property (atomic, readonly, nullable) NSString *unfilteredFamilyName;
@property (atomic, readonly, nullable) NSString *familyName;
@property (atomic, readonly, nullable) NSPersonNameComponents *nameComponents;
@property (atomic, readonly, nullable) NSString *fullName;
@property (atomic, readonly, nullable) NSString *bio;
@property (atomic, readonly, nullable) NSString *bioEmoji;
@property (atomic, readonly, nullable) NSString *username;
@property (atomic, readonly) BOOL isStoriesCapable;
@property (atomic, readonly, nullable) OWSUserProfileBadgeInfo *primaryBadge;
@property (atomic, readonly, nullable) NSArray<OWSUserProfileBadgeInfo *> *profileBadgeInfo;
@property (atomic, readonly, nullable) NSString *avatarUrlPath;
// This filename is relative to OWSProfileManager.profileAvatarsDirPath.
@property (atomic, readonly, nullable) NSString *avatarFileName;
@property (atomic, readonly, nullable) NSDate *lastFetchDate;
// This field reflects the last time we sent or
// received a message from this user.  It is coarse;
// we only update it every N hours.
@property (atomic, readonly, nullable) NSDate *lastMessagingDate;

+ (instancetype)new NS_UNAVAILABLE;
- (instancetype)init NS_UNAVAILABLE;
- (nullable instancetype)initWithCoder:(NSCoder *)coder NS_DESIGNATED_INITIALIZER;
- (instancetype)initWithUniqueId:(NSString *)uniqueId NS_UNAVAILABLE;
- (instancetype)initWithGrdbId:(int64_t)grdbId uniqueId:(NSString *)uniqueId NS_UNAVAILABLE;

// This initializer should only be called internally.
- (instancetype)initWithAddress:(SignalServiceAddress *)address NS_DESIGNATED_INITIALIZER;

@property (nonatomic, readonly) SignalServiceAddress *publicAddress;

// --- CODE GENERATION MARKER

// This snippet is generated by /Scripts/sds_codegen/sds_generate.py. Do not manually edit it, instead run `sds_codegen.sh`.

// clang-format off

- (instancetype)initWithGrdbId:(int64_t)grdbId
                      uniqueId:(NSString *)uniqueId
                  avatarFileName:(nullable NSString *)avatarFileName
                   avatarUrlPath:(nullable NSString *)avatarUrlPath
                             bio:(nullable NSString *)bio
                        bioEmoji:(nullable NSString *)bioEmoji
                      familyName:(nullable NSString *)familyName
                isStoriesCapable:(BOOL)isStoriesCapable
                   lastFetchDate:(nullable NSDate *)lastFetchDate
               lastMessagingDate:(nullable NSDate *)lastMessagingDate
                profileBadgeInfo:(nullable NSArray<OWSUserProfileBadgeInfo *> *)profileBadgeInfo
                      profileKey:(nullable OWSAES256Key *)profileKey
                     profileName:(nullable NSString *)profileName
            recipientPhoneNumber:(nullable NSString *)recipientPhoneNumber
                   recipientUUID:(nullable NSString *)recipientUUID
                        username:(nullable NSString *)username
NS_DESIGNATED_INITIALIZER NS_SWIFT_NAME(init(grdbId:uniqueId:avatarFileName:avatarUrlPath:bio:bioEmoji:familyName:isStoriesCapable:lastFetchDate:lastMessagingDate:profileBadgeInfo:profileKey:profileName:recipientPhoneNumber:recipientUUID:username:));

// clang-format on

// --- CODE GENERATION MARKER

@property (atomic, readonly, class) SignalServiceAddress *localProfileAddress;
@property (nonatomic, readonly, class) AnyUserProfileFinder *userProfileFinder;

+ (BOOL)isLocalProfileAddress:(SignalServiceAddress *)address;
+ (SignalServiceAddress *)resolveUserProfileAddress:(SignalServiceAddress *)address;
+ (SignalServiceAddress *)publicAddressForAddress:(SignalServiceAddress *)address;

+ (nullable OWSUserProfile *)getUserProfileForAddress:(SignalServiceAddress *)address
                                          transaction:(SDSAnyReadTransaction *)transaction;

+ (OWSUserProfile *)getOrBuildUserProfileForAddress:(SignalServiceAddress *)recipientId
                                        transaction:(SDSAnyWriteTransaction *)transaction;

+ (nullable OWSUserProfile *)userProfileForUsername:(NSString *)username
                                        transaction:(SDSAnyReadTransaction *)transaction;

+ (BOOL)localUserProfileExistsWithTransaction:(SDSAnyReadTransaction *)transaction;
- (void)loadBadgeContentWithTransaction:(SDSAnyReadTransaction *)transaction;

// For use by the OWSUserProfile extension only.
- (void)applyChanges:(UserProfileChanges *)changes
    userProfileWriter:(UserProfileWriter)userProfileWriter
          transaction:(SDSAnyWriteTransaction *)transaction
           completion:(nullable OWSUserProfileCompletion)completion;

#pragma mark - Profile Avatars Directory

+ (NSString *)profileAvatarFilepathWithFilename:(NSString *)filename;
+ (NSString *)legacyProfileAvatarsDirPath;
+ (NSString *)sharedDataProfileAvatarsDirPath;
+ (NSString *)profileAvatarsDirPath;
+ (void)resetProfileStorage;

+ (NSSet<NSString *> *)allProfileAvatarFilePathsWithTransaction:(SDSAnyReadTransaction *)transaction;

+ (void)mergeUserProfilesIfNecessaryForAddress:(SignalServiceAddress *)address
                                   transaction:(SDSAnyWriteTransaction *)transaction;

- (OWSUserProfile *)shallowCopy;

@end

@interface NSNull (OWSUserProfile) <OWSMaybeUserProfile>
@end

NS_ASSUME_NONNULL_END
